@@ -53,6 +53,8 @@ MainWindow::MainWindow(QWidget *parent) :
 
 	iOperation = OpNone;
 	iDimmer = new dimmer(this);
+
+	iReposAutoUpdating = false;
 }
 
 MainWindow::~MainWindow()
@@ -82,37 +84,67 @@ void MainWindow::on_btnRepos_clicked()
 
 void MainWindow::on_btnUpdate_clicked()
 {
-	if( iOperation != OpNone ) return;
-
 	busyDialog(true, tr("Operation in progress"), tr("Updating catalogs"));
-	iAptInterface->updateCatalogs();
+	if( !iAptInterface->updateCatalogs() )
+	{
+		ConfirmDialog d(false, this);
+		d.setText("Error","<b>Unable to perform operation</b><br>Can't write repository lists");
+		d.exec();
+	}
 }
 
 void MainWindow::on_btnListInstalled_clicked()
 {
 	if( iOperation != OpNone ) return;
-
 	iOperation = OpUninstall;
+
 	busyDialog(true, tr("Operation in progress"), tr("Reading package lists"));
-	iAptInterface->updatePackageList();
+	if( !iAptInterface->updatePackageList() )
+	{
+		ConfirmDialog d(false, this);
+		d.setText("Error","<b>Unable to perform operation</b><br>Can't write repository lists");
+		d.exec();
+	}
 }
 
 void MainWindow::on_btnListInstallable_clicked()
 {
 	if( iOperation != OpNone ) return;
-
 	iOperation = OpInstall;
+
+	if( iAptInterface->needRepoRefresh() ) {
+		iReposAutoUpdating = true;
+		on_btnUpdate_clicked();
+		return;
+	}
+
 	busyDialog(true, tr("Operation in progress"), tr("Reading package lists"));
-	iAptInterface->updatePackageList();
+	if( !iAptInterface->updatePackageList() )
+	{
+		ConfirmDialog d(false, this);
+		d.setText("Error","<b>Unable to perform operation</b><br>Can't write repository lists");
+		d.exec();
+	}
 }
 
 void MainWindow::on_btnUpgrade_clicked()
 {
 	if( iOperation != OpNone ) return;
-
 	iOperation = OpUpgrade;
+
+	if( iAptInterface->needRepoRefresh() ) {
+		iReposAutoUpdating = true;
+		on_btnUpdate_clicked();
+		return;
+	}
+
 	busyDialog(true, tr("Operation in progress"), tr("Reading package lists"));
-	iAptInterface->updatePackageList();
+	if( !iAptInterface->updatePackageList() )
+	{
+		ConfirmDialog d(false, this);
+		d.setText("Error","<b>Unable to perform operation</b><br>Can't write repository lists");
+		d.exec();
+	}
 }
 
 void MainWindow::operationFinished(AptInterface::aptMode lastMode_, bool success_, QString msg_)
@@ -193,6 +225,29 @@ void MainWindow::operationFinished(AptInterface::aptMode lastMode_, bool success
 		return;
 	}
 
+	if( lastMode_ == AptInterface::ModeAptGetUpdate && success_ && iReposAutoUpdating
+		&& ( iOperation==OpInstall || iOperation==OpUpgrade ) )
+	{
+		iReposAutoUpdating = false;
+		if( iOperation==OpInstall ) {
+			iOperation = OpNone;
+			on_btnListInstallable_clicked();
+			return;
+		}
+		if( iOperation==OpUpgrade ) {
+			iOperation = OpNone;
+			on_btnUpgrade_clicked();
+			return;
+		}
+	} else if( lastMode_ == AptInterface::ModeAptGetUpdate ) {
+		iOperation = OpNone;
+		busyDialog(false);
+		ConfirmDialog d(false, this);
+		d.setText("Operation finished","Repositories/catalogs updated");
+		d.exec();
+		return;
+	}
+
 	busyDialog(false);
 	if( !success_ ) {
 		ConfirmDialog d(false, this);
